@@ -122,7 +122,6 @@ static void print_libmsi_error(LibmsiResultError r)
     case LIBMSI_RESULT_CALL_NOT_IMPLEMENTED:
         fprintf(stderr, "%s: not implemented\n", program_name);
         exit(1);
-    case LIBMSI_RESULT_NO_MORE_ITEMS:
     case LIBMSI_RESULT_NOT_FOUND:
         fprintf(stderr, "%s: not found\n", program_name);
         exit(1);
@@ -180,7 +179,7 @@ static void print_strings_from_query(LibmsiQuery *query, GError **error)
         g_object_unref(rec);
     }
 
-    if (!g_error_matches(err, LIBMSI_RESULT_ERROR, LIBMSI_RESULT_NO_MORE_ITEMS))
+    if (err)
         g_propagate_error(error, err);
 
     g_clear_error(&err);
@@ -196,7 +195,7 @@ static int cmd_streams(struct Command *cmd, int argc, char **argv, GError **erro
         cmd_usage(stderr, cmd);
     }
 
-    db = libmsi_database_new(argv[1], LIBMSI_DB_OPEN_READONLY, error);
+    db = libmsi_database_new(argv[1], LIBMSI_DB_FLAGS_READONLY, NULL, error);
     if (!db)
         goto end;
 
@@ -230,7 +229,7 @@ static int cmd_tables(struct Command *cmd, int argc, char **argv, GError **error
         cmd_usage(stderr, cmd);
     }
 
-    db = libmsi_database_new(argv[1], LIBMSI_DB_OPEN_READONLY, error);
+    db = libmsi_database_new(argv[1], LIBMSI_DB_FLAGS_READONLY, NULL, error);
     if (!db)
         goto end;
 
@@ -319,7 +318,7 @@ static int cmd_suminfo(struct Command *cmd, int argc, char **argv, GError **erro
         cmd_usage(stderr, cmd);
     }
 
-    db = libmsi_database_new(argv[1], LIBMSI_DB_OPEN_READONLY, error);
+    db = libmsi_database_new(argv[1], LIBMSI_DB_FLAGS_READONLY, NULL, error);
     if (!db)
         goto end;
 
@@ -373,17 +372,16 @@ static int cmd_extract(struct Command *cmd, int argc, char **argv, GError **erro
     LibmsiDatabase *db = NULL;
     LibmsiQuery *query = NULL;
     LibmsiRecord *rec = NULL;
-    GOutputStream *out = NULL;
     GInputStream *in = NULL;
     int r = 1;
-    char *buf;
-    unsigned size, bufsize;
+    char buffer[4096];
+    size_t n_read, n_written;
 
     if (argc != 3) {
         cmd_usage(stderr, cmd);
     }
 
-    db = libmsi_database_new(argv[1], LIBMSI_DB_OPEN_READONLY, error);
+    db = libmsi_database_new(argv[1], LIBMSI_DB_FLAGS_READONLY, NULL, error);
     if (!db)
         goto end;
 
@@ -406,17 +404,21 @@ static int cmd_extract(struct Command *cmd, int argc, char **argv, GError **erro
     _setmode(STDOUT_FILENO, O_BINARY);
 #endif
 
-    out = g_unix_output_stream_new(STDOUT_FILENO, FALSE);
     in = G_INPUT_STREAM (libmsi_record_get_stream(rec, 1));
-    if (g_output_stream_splice(out, in, 0, NULL, error) == -1)
-        goto end;
+    for (;;) {
+        n_read = g_input_stream_read (in, buffer, sizeof (buffer), NULL, error);
+        if (n_read == -1)
+            goto end;
+        if (n_read == 0)
+            break;
+
+	full_write (STDOUT_FILENO, buffer, n_read);
+    }
 
     if (!*error)
         r = 0;
 
 end:
-    if (out)
-        g_object_unref(out);
     if (in)
         g_object_unref(in);
     if (rec)
@@ -640,7 +642,7 @@ static gboolean export_sql( LibmsiDatabase *db, const char *table, GError **erro
         }
     }
 
-    if (!g_error_matches(err, LIBMSI_RESULT_ERROR, LIBMSI_RESULT_NO_MORE_ITEMS)) {
+    if (err) {
         g_propagate_error(error, err);
         success = FALSE;
     }
@@ -670,7 +672,7 @@ static int cmd_export(struct Command *cmd, int argc, char **argv, GError **error
         cmd_usage(stderr, cmd);
     }
 
-    db = libmsi_database_new(argv[1], LIBMSI_DB_OPEN_READONLY, error);
+    db = libmsi_database_new(argv[1], LIBMSI_DB_FLAGS_READONLY, NULL, error);
     if (!db)
         return 1;
 
@@ -774,7 +776,10 @@ int main(int argc, char **argv)
     GError *error = NULL;
     struct Command *cmd = NULL;
 
-    g_type_init();
+#if !GLIB_CHECK_VERSION(2,35,1)
+    g_type_init ();
+#endif
+
     program_name = get_basename(argv[0]);
 
     if (argc == 1) {
